import numpy as np
import sys
import matplotlib.pyplot as plt
import util.file_handler as file_handler
import util.plotter_scroll as plotter_scroll
import api.sensor_net_map as net_map
import pysa.emd as emd
import pysa.utils as utils
import util.plotter as plotter


########################################################################################################################
#                                  Activity: CLEANING using automatic trial rejection                                  #
#                                                                                                                      #
#   This activity loads the EMD-filtered EEG data and applies automatic trial rejection function                       #
#   (see api.sensor_net_map.py). It can also check for bad electrodes, but these will be detected by the               #
#   trial rejection function.                                                                                          #
#                                                                                                                      #
########################################################################################################################

def start(username=''):

    # --------------------------------------USER INFORMATION AND PARAMETERS------------------------------------------- #
    if username == '':
        print "Please specify subject username in 'main.py'"
        sys.exit()

    # This does not have to be 'True', because the trial rejection function handles this as well.
    check_for_bad_electrodes = False
    # Should be 'True'
    check_for_bad_trials = True
    # Constant parameters (Specific to the experimental paradigm)
    samples_per_trial = 3000  # 6sec trial length
    sample_frequency = 500.0  # [Hz]

    # ---------------------------------------------------------------------------------------------------------------- #

    # -------------------------------------------FILENAMES------------------------------------------------------------ #
    filename_seg = username + '_data_motor_cortex_segmented.mat'
    filename_filtered_data = 'data_matrix_emd_filtered.mat'
    filename_line_noise_matrix = 'line_noise_matrix.mat'
    filename_trend_matrix = 'trend_matrix.mat'
    filename_bad_electrodes = 'bad_electrodes_motor_cortex.mat'
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------RAW EEG DATA---------------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_seg)
    # data_matrix_raw = data_json['data_seg']
    events = data_json['events']
    n_trials = len(events)
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------EMD-FILTERED DATA----------------------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_filtered_data)
    data_matrix_filtered = data_json[filename_filtered_data[:-4]]
    data_json = file_handler.read_mat_file_into_json_object(username, filename_filtered_data)
    data_matrix = data_json[filename_filtered_data[:-4]]
    data_json = file_handler.read_mat_file_into_json_object(username, filename_filtered_data)
    data_matrix_ar = data_json[filename_filtered_data[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ------------------------------------------TREND-MATRIX and LINE NOISE DATA-------------------------------------- #
    data_json = file_handler.read_mat_file_into_json_object(username, filename_trend_matrix)
    trend_matrix = data_json[filename_trend_matrix[:-4]]

    data_json = file_handler.read_mat_file_into_json_object(username, filename_line_noise_matrix)
    line_noise_matrix = data_json[filename_line_noise_matrix[:-4]]
    # ---------------------------------------------------------------------------------------------------------------- #

    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #
    # ---------------------------------------------------------------------------------------------------------------- #

    # -----------------------------------------------TRIAL REJECTION-------------------------------------------------- #
    print("\x1b[2J\x1b[H")
    introduction = "EEG automatic electrode- and trial-rejection - Subject 3 " # + username.title()
    print "%s\n%s\n%s" % ('-' * len(introduction), introduction, '-' * len(introduction))

    if check_for_bad_electrodes:
        print "Checking for bad electrodes..."
        # Clean the data (checks only bad channels, no artifact-correction)
        data_matrix_filtered, bad_electrodes = net_map.check_for_bad_electrodes(data_matrix_filtered, threshold=750.0,
                                                                                n_trials=n_trials)
    # The parameters 'threshold' and 'inter_trial_step' may have to be tuned for each subject.
    if check_for_bad_trials:
        print "Checking for bad trials..."
        data_matrix_checked, bad_trials, bad_trials_matrix = \
            net_map.check_for_bad_trials(data_matrix, threshold=50, n_trials=n_trials, inter_trial_step=100)

        for el in range(len(bad_trials_matrix)):
            for trial in range(len(bad_trials_matrix[el])):
                if bad_trials_matrix[el, trial]:
                    if trial >= 1:
                        data = data_matrix_checked[el, (trial-1)*samples_per_trial:trial*samples_per_trial]
                    else:
                        data = data_matrix_checked[el, trial*samples_per_trial:(trial+1)*samples_per_trial]
                    data_matrix_checked[el, trial*samples_per_trial:(trial+1)*samples_per_trial] = 1.0*np.mean(data)

        file_handler.save_experiment_data_as_mat(data_matrix_checked, 'data_matrix_cleaned', username,
                                                 'data_matrix_cleaned.mat')
        file_handler.save_experiment_data_as_mat(bad_trials_matrix, 'bad_trials_matrix', username,
                                                 'bad_trials_matrix.mat')
        print bad_trials
    else:
        data_matrix_checked = data_matrix_ar

    print "Trial rejection process finished. Plotting the cleaned EEG data..."

    # -------------------------------------------------PLOTS---------------------------------------------------------- #

    plotter_scroll.plot_data_sliding(data_matrix_filtered, events, sample_frequency, title='Filtered data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)
    plotter_scroll.plot_data_sliding(data_matrix_checked, events, sample_frequency, title='Checked data',
                                     n_sec_per_trial=6, n_trial_per_frame=2)

    try:
        plt.draw()
        plt.show(block=True)
    except KeyboardInterrupt, ki:
        sys.exit()
    # ---------------------------------------------------------------------------------------------------------------- #
