from __future__ import print_function
import time
from sklearn import svm
from sklearn.tree import DecisionTreeClassifier
from sklearn.neighbors import KNeighborsClassifier
from sklearn.ensemble import RandomForestClassifier
from sklearn.ensemble import AdaBoostClassifier
from sklearn.naive_bayes import GaussianNB
from sklearn.lda import LDA
from sklearn import linear_model
from sklearn import metrics
from sklearn.neural_network import BernoulliRBM
from sklearn.pipeline import Pipeline
from sklearn.grid_search import GridSearchCV
from sklearn import cross_validation


def grid_search(classifier_name, trained_data=(), parameters={}):
    trained_set = trained_data[0]
    trained_classes = trained_data[1]
    classifier = svm.SVC(kernel=classifier_name)
    classifier = GridSearchCV(classifier, parameters)
    classifier.fit(trained_set, trained_classes)
    cross_validation_score = cross_validation.cross_val_score(classifier, trained_set, trained_classes, cv=5)
    best_parameters = classifier.best_estimator_.get_params()
    params_keys = parameters.keys()

    return classifier, best_parameters


def predict_test_data(classifier, test_data=()):
    test_set = test_data[0]
    test_classes = test_data[1]
    predicted = classifier.predict(test_set)
    score = get_score(test_classes, predicted)
    return score


def map_classifier_name(classifier_name):
    if classifier_name == 'rbf' or classifier_name == 'linear' or classifier_name == 'poly' or classifier_name == 'sigmoid':
        return svm.SVC(kernel=classifier_name)
    elif classifier_name == 'nearest_neighbors':
        return KNeighborsClassifier()
    elif classifier_name == 'decision_tree':
        return DecisionTreeClassifier()
    elif classifier_name == 'random_forest':
        return RandomForestClassifier()
    elif classifier_name == 'ada_boost':
        return AdaBoostClassifier()
    elif classifier_name == 'naive_bayes':
        return GaussianNB()
    elif classifier_name == 'lda':
        return LDA()


def rbf_svm(trained_set, test_set, trained_classes, test_classes, feature_selector=(), parameters={}):
    start_time = time.time()
    name = "RBF SVM"
    feature_selector_name = ""
    classifier = svm.SVC(parameters)

    if feature_selector:
        classifier = Pipeline([feature_selector, (name, classifier)])
        feature_selector_name = feature_selector[0]

    train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time)


def train_and_predict_data(name, feature_selector_name, classifier, trained_set, test_set, trained_classes, test_classes, start_time):
    classifier.fit(trained_set, trained_classes)
    score = classifier.score(test_set, test_classes)
    cross_validation_score = cross_validation.cross_val_score(classifier, trained_set, trained_classes, cv=5)
    predicted = classifier.predict(test_set)

    if feature_selector_name:
        selected_features = classifier.named_steps[feature_selector_name].get_support()
    else:
        selected_features = "No Feature Selector"

    get_score(test_classes, predicted)


def get_score(test_classes, predicted):
    score = metrics.accuracy_score(test_classes, predicted)
    return score*100
