import scipy
import matplotlib.pyplot as plt
import numpy as np
import pysa.utils as utils


# -------------------------------------------------------------------------------------------------------------------- #
#                                                ERD/ERS PLOTTING                                                  #
# -------------------------------------------------------------------------------------------------------------------- #

def plot_avg_erd(data, electrode='', event='', stimulus_onset=250):
    n_dim = False
    if len(np.shape(data)) > 1:
        n_imfs, trial_length = np.shape(data)
        if n_imfs > 1:
            n_dim = True
    else:
        trial_length = len(data)
        n_imfs = 1

    time_axis = np.linspace(0, (2*trial_length)-2, trial_length)
    fig, axarr = plt.subplots(n_imfs, 1, sharex=True)
    fig.suptitle(electrode, fontsize=14)
    for i in range(n_imfs):
        erds = data[i]
        if n_dim:
            axarr[i].plot(time_axis, erds, linewidth=2.0, label='ERD')
            axarr[i].grid()
            axarr[i].axvline(stimulus_onset*2, color='k', linestyle='--', linewidth=2.0)
            axarr[i].plot(time_axis, np.ones(len(time_axis))*np.mean(erds[stimulus_onset+50:1400]), color='r', linestyle='--',
                          linewidth=1.5, label='mean ERD')
            electrode = 'IMF #' + str(i+1)
            ylabel_str = 'ERD [%]'
            axarr[i].set_title(electrode, fontsize=10)
            axarr[i].set_ylabel(ylabel_str)

        else:
            axarr.plot(time_axis, erds, linewidth=2.0, label='ERD')
            axarr.grid()
            axarr.axvline(stimulus_onset*2, color='k', linestyle='--', linewidth=2.0)
            axarr.plot(time_axis, np.ones(len(time_axis))*np.mean(erds[stimulus_onset+50:1400]), color='r', linestyle='--',
                       linewidth=1.5, label='average ERD')
            electrode = 'IMF #' + str(i+1)
            ylabel_str = 'ERD [%]'
            axarr.set_title(electrode, fontsize=10)
            axarr.set_ylabel(ylabel_str)

    if n_dim:
        legend = axarr[0].legend(loc='upper right', shadow=True)
        axarr[-1].set_xlabel('Time [ms]')
        ax2 = axarr[0].twiny()
        ax2.set_xlim(axarr[0].get_xlim())

    else:
        legend = axarr.legend(loc='upper center', shadow=True)
        axarr.set_xlabel('Time [s]')

        ax2 = axarr.twiny()
        ax2.set_xlim(axarr.get_xlim())

    ax2.set_xticks([stimulus_onset])
    ax2.set_xticklabels([event])

    # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
    legend_frame = legend.get_frame()
    legend_frame.set_facecolor('0.90')

    # Set the fontsize
    for label in legend.get_texts():
        label.set_fontsize('large')

    for label in legend.get_lines():
        label.set_linewidth(1.5)  # the legend line width
    plt.subplots_adjust(left=0.11, right=0.96, bottom=0.10, top=0.91, wspace=0.1, hspace=0.37)


def plot_erds(data, electrode='', event='', stimulus_onset=250):
    dim = 1
    if len(np.shape(data)) == 3:
        print np.shape(data)
        n_electrodes, n_imfs, trial_length = np.shape(data)
        if n_imfs == 1:
            data = data[0, 0]
        else:
            data = data[0, :]
    elif len(np.shape(data)) == 2:
        n_imfs, trial_length = np.shape(data)
        if n_imfs == 1:
            data = data[0]
    else:
        n_imfs = 1
        trial_length = len(data)

    time_axis = np.linspace(0, (2*trial_length)-2, trial_length)
    fig, axarr = plt.subplots(n_imfs, 1, sharex=True)
    suptitle = 'Electrode: ' + electrode
    fig.suptitle(suptitle, fontsize=14)
    for i in range(n_imfs):
        if n_imfs >= 2:
            min_data = min(data[i])
            max_data = max(data[i])
            power = utils.normalize_data(data[i], min_data, max_data, trial_length)**2
        else:
            min_data = min(data)
            max_data = max(data)
            power = utils.normalize_data(data, min_data, max_data, trial_length)**2
        baseline_length = stimulus_onset - 0
        mean_baseline_power = float(np.mean(power[:baseline_length]))
        erds = (100.0*(power - mean_baseline_power))/mean_baseline_power
        erd_limit = 300.0
        for k in range(len(erds)):
            if erds[k] > erd_limit:
                erds[k] = erd_limit
            elif erds[k] < -erd_limit:
                erds[k] = -erd_limit

        if n_imfs >= 2:
            axarr[i].plot(time_axis, erds, linewidth=2.0, label='ERD')
            axarr[i].grid()
            axarr[i].axvline(stimulus_onset*2, color='k', linestyle='--', linewidth=2.0)
            axarr[i].axhline(y=np.mean(erds[:350]), xmin=0.0, xmax=0.175, color='g', linestyle='--', linewidth=2.0, label='Mean ERD prior to stimuli')
            axarr[i].axhline(y=np.mean(erds[350:1700]), xmin=705.0/4000.0, xmax=3405.0/4000.0, color='r', linestyle='--', linewidth=2.0, label='Mean ERD during movement')
            axarr[i].axhline(y=np.mean(erds[1700:]), xmin=3400.0/4000.0, xmax=1.0, color='m', linestyle='--', linewidth=2.0, label='Post-movement mean ERD')

            # axarr[i].plot(time_axis, np.ones(len(time_axis))*np.mean(erds[baseline_length:]), color='r', linestyle='--', linewidth=1.5,
            #               label='mean ERD')
            electrode = 'IMF #' + str(i+1)
            ylabel_str = 'ERD [%]'
            axarr[i].set_title(electrode, fontsize=10)
            axarr[i].set_ylabel(ylabel_str)

        else:
            axarr.plot(time_axis, erds, linewidth=2.0, label='ERD')
            axarr.grid()
            axarr.axvline(stimulus_onset*2, color='k', linestyle='--', linewidth=2.0)
            axarr.axhline(y=np.mean(erds[:250]), xmin=0.0, xmax=500.0/4000.0, color='g', linestyle='--', linewidth=2.5, label='Mean ERD prior to stimuli')
            axarr.axhline(y=np.mean(erds[350:1700]), xmin=700.0/4000.0, xmax=3400.0/4000.0, color='r', linestyle='--', linewidth=2.5, label='Mean ERD during movement')
            axarr.axhline(y=np.mean(erds[1700:]), xmin=3400.0/4000.0, xmax=1.0, color='c', linestyle='--', linewidth=2.5, label='Post-movement mean ERD')
            # axarr.plot(time_axis, np.ones(len(time_axis))*np.mean(erds), color='r', linestyle='--', linewidth=2.0,
            #            label='Mean ERD')
            electrode = 'IMF #' + str(i+2)
            ylabel_str = 'ERD [%]'
            axarr.set_title(electrode, fontsize=10)
            axarr.set_ylabel(ylabel_str)

    if n_imfs >= 2:
        legend = axarr[0].legend(loc='upper right', shadow=True)
        axarr[-1].set_xlabel('Time [ms]')
        ax2 = axarr[0].twiny()
        ax2.set_xlim(axarr[0].get_xlim())

    else:
        legend = axarr.legend(loc='upper center', shadow=True)
        axarr.set_xlabel('Time [ms]')

        ax2 = axarr.twiny()
        ax2.set_xlim(axarr.get_xlim())

    ax2.set_xticks([stimulus_onset*2])
    ax2.set_xticklabels([event])

    # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
    legend_frame = legend.get_frame()
    legend_frame.set_facecolor('0.90')

    # Set the fontsize
    for label in legend.get_texts():
        label.set_fontsize('large')

    for label in legend.get_lines():
        label.set_linewidth(1.5)  # the legend line width
    plt.subplots_adjust(left=0.11, right=0.96, bottom=0.10, top=0.91, wspace=0.1, hspace=0.37)


def plot_features(feature_matrix, feature_names, use_mean=False, offset=0.0, plot=plt):

    n_features, n_trials = np.shape(feature_matrix)

    xdata = np.linspace(0, n_trials, n_trials)

    # set up figure
    fig = plot.figure()
    plot.subplots_adjust(left=0.03, bottom=0.09, right=0.99, top=0.96)
    ax = fig.add_subplot(111, axisbg='lightgoldenrodyellow')
    ax.autoscale(True)
    ax.set_title('Features')
    ax.set_xlabel('Trial #')

    # plot data set
    lns = []

    for feat in range(n_features + 1):

        if feat == n_features and n_features == 2:
            lab3 = feature_names[feat-2] + '-' + lab
            ln, = ax.plot(xdata, feature_matrix[feat-2, :] - data, label=lab3)
            lab4 = lab3 + '_mean'
            ln, = ax.plot(np.ones(len(data))*np.mean(feature_matrix[feat-2, :] - data), label=lab4)
            lns.append(ln)
        else:
            data = feature_matrix[feat, :]

            lab = feature_names[feat]
            # ln, = ax.plot(xdata, data + (feat*offset), label=lab)
            if use_mean:
                lab2 = lab + '_mean'
                # ln, = ax.plot(np.ones(len(data))*np.mean(data), label=lab2)

            # lns.append(ln)

    legend = ax.legend(loc='upper left', shadow=True)
    # The frame is matplotlib.patches.Rectangle instance surrounding the legend.
    legend_frame = legend.get_frame()
    legend_frame.set_facecolor('0.90')

    # Set the fontsize
    for label in legend.get_texts():
        label.set_fontsize('large')
    for label in legend.get_lines():
        label.set_linewidth(1.5)  # the legend line width


# -------------------------------------------------------------------------------------------------------------------- #
#                                                HHT PLOTTING                                                          #
# -------------------------------------------------------------------------------------------------------------------- #


# FUNCTION FOR PLOTTING THE HILBERT SPECTRA FOR IMFs
def plot_hilbert_spectra_multiple_imfs(time, frequency, amplitude, title, plotter=plt, fs=500, n_levels=20, events=[0],
                         stimulus_onset_in_ms=500, baseline_correct=False):

    # Scale factor (to plot frequency with decimal precision)
    scale_freq = 1
    # Defining minimum number of samples needed to define the frequency accurately, n:
    n = 15
    # Max scaled frequency
    max_freq = int(np.ceil(scale_freq*fs/n))

    # Creating time axis
    time_ax = np.linspace(0, len(time)-1, len(time))
    # Allocating memory for the rounded frequency and power array
    freq_rounded_array = np.zeros(np.shape(frequency), np.int)
    power_array = np.zeros(np.shape(amplitude), np.float)

    # Create GRID based on time axis and maximum frequency
    yi = np.linspace(0, max_freq, max_freq + 1)
    for k in range(len(amplitude)):

        if baseline_correct:
            min_data = min(amplitude[k])
            max_data = max(amplitude[k])
            n_samples = len(amplitude[k])
            power_array[k] = utils.normalize_data(amplitude[k], min_data, max_data, n_samples)**2
            baseline_length = int(stimulus_onset_in_ms*0.5) - 50
            baseline_mean_power = float(np.mean(power_array[k, :baseline_length]))
            power_array[k] = 100*(power_array[k] - baseline_mean_power) / baseline_mean_power
            max_erds = 100.0
            for i in range(len(time_ax)):
                if power_array[k, i] > max_erds:
                    power_array[k, i] = max_erds
                elif power_array[k, i] < -max_erds:
                    power_array[k, i] = -max_erds

            min_data = min(power_array[k])
            max_data = max(power_array[k])
            n_samples = len(power_array[k])
            power_array[k] = utils.normalize_data(power_array[k], min_data, max_data, n_samples)

        else:
            min_data = min(amplitude[k])
            max_data = max(amplitude[k])
            n_samples = len(amplitude[k])
            power_array[k] = utils.normalize_data(amplitude[k], min_data, max_data, n_samples)**2

    # Z = np.ones((max_freq + 1, len(time_ax)), np.float)*-100.0
    Z = np.zeros((max_freq + 1, len(time_ax)), np.float)

    X, Y = np.meshgrid(time_ax, yi)

    for k in range(len(amplitude)):

        # Round the frequency to the nearest (results in OK resolution if scale_freq > 1, eg scale_freq=10)
        freq_rounded_array[k] = np.ceil(frequency[k]*scale_freq)
        # Compute the power, and add it to the previous if the same inst. frequency exists.
        for i in range(len(time_ax)):
            if freq_rounded_array[k, i] >= max_freq:
                freq_rounded_array[k, i] = max_freq - 1
            power = power_array[k, i]

            Z[int(freq_rounded_array[k, i]), int(time_ax[i])] = power

    # Create figure and subplot.
    # Set titles and labels.
    fig = plotter.figure()
    plt.subplots_adjust(left=0.08, bottom=0.10, right=1.00, top=0.94)
    suptitle = 'Electrode: ' + title
    fig.suptitle(suptitle, fontsize=14)
    ax = plotter.subplot(111)
    ax.set_xlabel('Time [ms]')
    ax.set_ylabel('Frequency [Hz]')

    # Create contour plot og time, frequency and logarithmic power. Scale frequencies back to original values.
    cax = ax.contourf(X*2, Y/scale_freq, Z, n_levels)
    # Assign color bar to the contour plot
    cb = fig.colorbar(cax)
    # Set label and draw plot
    lab = 'Normalized Power'
    cb.set_label(lab)
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())

    if events[0] != 0:
        labels = []
        for k in range(len(events)):
            ax2.set_xticks([k+stimulus_onset_in_ms])
            la = 'Stm: ' + events[k]
            labels.append(la)
            ax2.axvline(k+stimulus_onset_in_ms, color='w', linestyle='--')
        ax2.set_xticklabels(labels)
    else:
        ax2.set_xticks([stimulus_onset_in_ms])
        ax2.set_xticklabels(['stm'])
        ax2.axvline(stimulus_onset_in_ms, color='w', linestyle='--')
    plotter.draw()


# FUNCTION FOR PLOTTING THE HILBERT SPECTRA FOR IMFs
def plot_hilbert_spectra_power(time, frequency, amplitude, title, plotter=plt, fs=500, n_levels=180, events=[0],
                         stimulus_onset_in_ms=500, baseline_correct=False):

    # Scale factor (to plot frequency with decimal precision)
    scale_freq = 10.0
    # Defining minimum number of samples needed to define the frequency accurately, n:
    n = 10.0
    # Max scaled frequency
    max_freq = int(scale_freq*fs/n)

    # Creating time axis
    time_ax = np.linspace(0, len(time)-1, len(time))
    # Allocating memory for the rounded frequency
    freq_rounded_array = np.zeros(np.shape(frequency), np.int)

    # Create GRID based on time axis and maximum frequency
    yi = np.linspace(0, max_freq, max_freq + 1)

    if baseline_correct:
        if isinstance(frequency[0], np.ndarray):
            for k in range(len(amplitude)):
                amplitude[k] /= np.mean(amplitude[k, :int(fs*stimulus_onset_in_ms/1000)])

    absolute_min_power = 100.0
    if isinstance(frequency[0], np.ndarray):
        for k in range(len(frequency)):
            min_data = amplitude[k]
            max_data = amplitude[k]
            n_samples = len(amplitude[k])

            amplitude[k] = utils.normalize_data(amplitude[k], min_data, max_data, n_samples)
            min_power = min(amplitude[k]*amplitude[k])

            if min_power < absolute_min_power:
                absolute_min_power = min_power

    Z = np.ones((max_freq + 1, len(time_ax)))*absolute_min_power
    print absolute_min_power
    X, Y = np.meshgrid(time_ax, yi)
    power_array = np.zeros(np.shape(time))

    # Enter loop if more than one IMF exists
    if isinstance(frequency[0], np.ndarray):
        print 'Plotting multiple IMFs in Hilbert Spectra.'
        for i in range(len(frequency)):

            # Round the frequency to the nearest (results in OK resolution if scale_freq > 1, eg scale_freq=10)
            freq_rounded_array[i] = np.ceil(frequency[i]*scale_freq)
            # Compute the power, and add it to the previous if the same inst. frequency exists.
            for k in range(len(time_ax)):
                if freq_rounded_array[i, k] >= max_freq:
                    freq_rounded_array[i, k] = max_freq - 1
                    amplitude[i, k] = 0.0

                power = amplitude[i, k]*amplitude[i, k]

                current_power = Z[int(freq_rounded_array[i, k]), int(time_ax[k])]
                if current_power > absolute_min_power:  # + 0.15:
                    # print "Crossing"
                    Z[int(freq_rounded_array[i, k]), int(time_ax[k])] = current_power + power
                else:
                    Z[int(freq_rounded_array[i, k]), int(time_ax[k])] = power

                power_array[k] = Z[int(freq_rounded_array[i, k]), int(time_ax[k])]
    else:

        # Round the frequency to the nearest (results in OK resolution if scale_freq > 1, eg scale_freq=10)
        freq_rounded_array = np.ceil(frequency*scale_freq)
        # Compute the logarithmic power, and add it to the previous if the same inst. frequency exists.
        for k in range(len(time_ax)):

            if freq_rounded_array[k] >= max_freq:
                freq_rounded_array[k] = max_freq - 1
                amplitude[k] = 0.0
            Z[int(freq_rounded_array[k]), int(time_ax[k])] = amplitude[k]*amplitude[k]

    # Create figure and subplot.
    # Set titles and labels.
    fig = plotter.figure()
    suptitle = title
    fig.suptitle(suptitle)
    ax = plotter.subplot(111)
    ax.set_xlabel('Time [ms]')
    ax.set_ylabel('Frequency [Hz]')

    # Create contour plot og time, frequency and logarithmic power. Scale frequencies back to original values.
    cax = ax.contourf(X*2, Y/scale_freq, Z, n_levels)
    # Assign color bar to the contour plot
    cb = fig.colorbar(cax)
    # Set label and draw plot
    lab = "%s [%s$V^2$]" % ('Normalized Power', u'\u00b5')
    cb.set_label(lab)
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())
    # ax2.set_xlabel('Event triggers')

    if events[0] != 0:
        labels = []
        for k in range(len(events)):
            ax2.set_xticks([k+stimulus_onset_in_ms])
            labels.append(events[k])
            ax2.axvline(k+stimulus_onset_in_ms, color='w', linestyle='--')
        ax2.set_xticklabels(labels)
    else:
        ax2.set_xticks([stimulus_onset_in_ms])
        ax2.set_xticklabels(['stm'])
        ax2.axvline(stimulus_onset_in_ms, color='w', linestyle='--')
    plotter.draw()


# -------------------------------------------------------------------------------------------------------------------- #
#                                                EMD-related PLOTTING                                                  #
# -------------------------------------------------------------------------------------------------------------------- #

def plot_intrinsic_mode_functions(imfs, time_axis, channel=str, plotter=plt, residue=[0],
                                  is_zero_indexed=True):

    n_rows = len(imfs)
    if len(residue) == 1:
        f, axis = plotter.subplots(n_rows, 1, sharex=True, sharey=False)
    else:
        f, axis = plotter.subplots(n_rows + 1, 1, sharex=True, sharey=False)

    sup_title = "Channel " + channel
    f.suptitle(sup_title, fontsize=18)

    for i in range(0, n_rows):
        title = 'IMF: ' + str(i+1)
        if is_zero_indexed:
            axis[i].plot(time_axis, imfs[i])
        else:
            axis[i].plot(time_axis, imfs[i+1])
        axis[i].set_title(title)
        # axis[i].set_ylim([-20, 20])
        axis[i].grid()
    if len(residue) != 1:
        axis[n_rows].plot(time_axis, residue)
        axis[n_rows].set_title("Residue")
        axis[n_rows].grid()
        axis[n_rows].set_xlabel('Time [s]')
    f.subplots_adjust(hspace=.5)


def plot_intrinsic_mode_functions_with_frequency_and_amplitude(imfs, time_axis, freq_array, amp_array,
                                                               channel=str, plotter=plt, is_zero_indexed=True):
    n_rows = len(imfs)
    sup_title = "Channel " + channel
    f, axis = plotter.subplots(n_rows, 2, sharex=False, sharey=False)
    f.suptitle(sup_title, fontsize=18)

    for i in range(0, n_rows):
        # Maybe for-loop iterating through all imf inst. frequencies here
        freq_hz_sorted, amplitude_sorted = (list(t) for t in zip(*sorted(zip(freq_array[i], amp_array[i]))))

        title = 'IMF: ' + str(i+1)
        if is_zero_indexed:
            axis[i][0].plot(time_axis, imfs[i])
        else:
            axis[i][0].plot(time_axis, imfs[i+1])
        axis[i][0].set_title(title)
        axis[i][0].grid()
        axis[i][1].plot(freq_hz_sorted, amplitude_sorted)
        axis[i][1].set_ylabel('Amplitude')
        # axis[i][1].set_xlim([4, 40])
        axis[i][1].grid()
    axis[n_rows - 1][0].set_xlabel('Time [s]')
    axis[n_rows - 1][1].set_xlabel('Frequency [Hz]')
    f.subplots_adjust(hspace=.5)


def plot_original_signal_from_intrinsic_mode_functions(imfs, residue, samples_per_second, channel, plotter=plt):
    n_rows = len(imfs)
    n_samples = len(imfs[1])
    final_signal = scipy.zeros(len(imfs[1]))
    time_axis = scipy.linspace(start=0, stop=n_samples / samples_per_second, num=n_samples)

    for i in range(n_rows):
        final_signal = scipy.add(final_signal, imfs[i + 1])

    final_signal = scipy.add(final_signal, residue)
    f, axis = plotter.subplots(1, 1)
    sup_title = "Channel " + channel
    f.suptitle(sup_title, fontsize=18)
    axis.plot(time_axis, final_signal)
    axis.grid()


# -------------------------------------------------------------------------------------------------------------------- #
#                                                OLD PLOTTING FUNCTIONS                                                #
# -------------------------------------------------------------------------------------------------------------------- #


def plot_single_channel(ch_data, my_plot=plt, filename=str, sample_frequency=500):
    my_plot.ion()
    f, ax = my_plot.subplots(1, 1)
    f.suptitle(filename)
    my_plot.subplots_adjust(left=0.04, right=0.98, bottom=0.04, top=0.92, wspace=0.1, hspace=0.38)
    time_axis = scipy.linspace(start=0, stop=len(ch_data) / sample_frequency, num=len(ch_data))
    ylabel_str = "%s [%sV]" % ('EEG', u'\u00b5')
    ax.set_ylabel(ylabel_str)
    ax.set_xlabel('Time [s]')
    ax.plot(time_axis, ch_data)
    # x_ticks = np.array([0, 2, 6])
    # my_xticks = ['Fix+', 'Stim', 'End']
    # my_plot.xticks(x_ticks, my_xticks)
    ax2 = ax.twiny()
    ax2.set_xlim(ax.get_xlim())
    ax2.set_xlabel('Event triggers')
    ax2.set_xticks([1])
    ax2.set_xticklabels(['stm'])
    ax.axvline(1, color='r', linestyle='--')
    my_plot.draw()


def plot_power_spectral_density(frequency, amplitude, plotter=plt, title=str):
    fig = plotter.figure()
    ax = fig.add_subplot(111)
    fig.suptitle(title)
    # frequency, amplitude = (list(t) for t in zip(*sorted(zip(frequency, amplitude))))
    power = np.multiply(amplitude, amplitude)
    ax.plot(frequency, power)
    ax.set_xlabel('Frequency [Hz]')
    ylabel = 'Power Spectral Density [%sV^2]' % u'\u00b5'
    ax.set_ylabel(ylabel)
    plotter.draw()


def plot_channel_pair(ch1, ch2, data_ch1, data_ch2, time_axis, title, plotter=plt):
    fig, axarr = plotter.subplots(2, 1, sharex=True)
    fig.suptitle(title)
    plotter.subplots_adjust(left=0.08, right=0.99, bottom=0.04, top=0.95, wspace=0.1, hspace=0.38)
    ylabel_str = "%s [%sV]" % ('EEG', u'\u00b5')

    axarr[0].plot(time_axis, data_ch1, label=ch1)
    axarr[0].grid()
    axarr[0].set_xlabel('Time [s]')
    axarr[0].set_ylabel(ylabel_str)
    axarr[0].set_title(ch1)
    axarr[1].plot(time_axis, data_ch2, label=ch2)
    axarr[1].grid()
    axarr[1].set_xlabel('Time [s]')
    axarr[1].set_ylabel(ylabel_str)
    axarr[1].set_title(ch2)

    plotter.draw()
